# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_agent_test_response_model import CreateAgentTestResponseModel
from ...types.get_tests_page_response_model import GetTestsPageResponseModel
from ...types.get_tests_summaries_by_ids_response_model import GetTestsSummariesByIdsResponseModel
from ...types.test_sharing_mode import TestSharingMode
from ...types.test_type import TestType
from .raw_client import AsyncRawTestsClient, RawTestsClient
from .types.tests_create_request_body import TestsCreateRequestBody
from .types.tests_get_response import TestsGetResponse
from .types.tests_list_request_sort_mode import TestsListRequestSortMode
from .types.tests_update_request_body import TestsUpdateRequestBody
from .types.tests_update_response import TestsUpdateResponse

if typing.TYPE_CHECKING:
    from .folders.client import AsyncFoldersClient, FoldersClient
    from .invocations.client import AsyncInvocationsClient, InvocationsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TestsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTestsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._folders: typing.Optional[FoldersClient] = None
        self._invocations: typing.Optional[InvocationsClient] = None

    @property
    def with_raw_response(self) -> RawTestsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTestsClient
        """
        return self._raw_client

    def create(
        self, *, request: TestsCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateAgentTestResponseModel:
        """
        Creates a new agent response test.

        Parameters
        ----------
        request : TestsCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateAgentTestResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs
        from elevenlabs.conversational_ai.tests import TestsCreateRequestBody_Llm

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.create(
            request=TestsCreateRequestBody_Llm(
                name="name",
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def move(
        self,
        *,
        entity_ids: typing.Sequence[str],
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Any:
        """
        Moves multiple tests or folders from one folder to another.

        Parameters
        ----------
        entity_ids : typing.Sequence[str]
            The IDs of tests or folders to move.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Tests or folders successfully moved to another folder

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.move(
            entity_ids=["entity_ids"],
        )
        """
        _response = self._raw_client.move(entity_ids=entity_ids, move_to=move_to, request_options=request_options)
        return _response.data

    def get(self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> TestsGetResponse:
        """
        Gets an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestsGetResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.get(
            test_id="TeaqRRdTcIfIu2i7BYfT",
        )
        """
        _response = self._raw_client.get(test_id, request_options=request_options)
        return _response.data

    def update(
        self, test_id: str, *, request: TestsUpdateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> TestsUpdateResponse:
        """
        Updates an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request : TestsUpdateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestsUpdateResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs
        from elevenlabs.conversational_ai.tests import TestsUpdateRequestBody_Llm

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.update(
            test_id="TeaqRRdTcIfIu2i7BYfT",
            request=TestsUpdateRequestBody_Llm(
                name="name",
            ),
        )
        """
        _response = self._raw_client.update(test_id, request=request, request_options=request_options)
        return _response.data

    def delete(self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Any:
        """
        Deletes an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.delete(
            test_id="TeaqRRdTcIfIu2i7BYfT",
        )
        """
        _response = self._raw_client.delete(test_id, request_options=request_options)
        return _response.data

    def summaries(
        self, *, test_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> GetTestsSummariesByIdsResponseModel:
        """
        Gets multiple agent response tests by their IDs. Returns a dictionary mapping test IDs to test summaries.

        Parameters
        ----------
        test_ids : typing.Sequence[str]
            List of test IDs to fetch. No duplicates allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTestsSummariesByIdsResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.summaries(
            test_ids=["test_id_1", "test_id_2"],
        )
        """
        _response = self._raw_client.summaries(test_ids=test_ids, request_options=request_options)
        return _response.data

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        parent_folder_id: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]] = None,
        include_folders: typing.Optional[bool] = None,
        sort_mode: typing.Optional[TestsListRequestSortMode] = None,
        sharing_mode: typing.Optional[TestSharingMode] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetTestsPageResponseModel:
        """
        Lists all agent response tests with pagination support and optional search filtering.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many Tests to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            Search query to filter tests by name.

        parent_folder_id : typing.Optional[str]
            Filter by parent folder ID. Use 'root' to get items in the root folder.

        types : typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]]
            If present, the endpoint will return only tests/folders of the given types.

        include_folders : typing.Optional[bool]
            Deprecated. Use the `types` query param and include `folder` instead.

        sort_mode : typing.Optional[TestsListRequestSortMode]
            Sort mode for listing tests. Use 'folders_first' to place folders before tests.

        sharing_mode : typing.Optional[TestSharingMode]
            Filter test visibility. Use `shared_with_me` to return only tests/folders shared with the current user that they did not create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTestsPageResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.tests.list(
            cursor="cursor",
            page_size=1,
            search="search",
            parent_folder_id="parent_folder_id",
            types=["llm"],
            include_folders=True,
            sort_mode="default",
            sharing_mode="all",
        )
        """
        _response = self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            search=search,
            parent_folder_id=parent_folder_id,
            types=types,
            include_folders=include_folders,
            sort_mode=sort_mode,
            sharing_mode=sharing_mode,
            request_options=request_options,
        )
        return _response.data

    @property
    def folders(self):
        if self._folders is None:
            from .folders.client import FoldersClient  # noqa: E402

            self._folders = FoldersClient(client_wrapper=self._client_wrapper)
        return self._folders

    @property
    def invocations(self):
        if self._invocations is None:
            from .invocations.client import InvocationsClient  # noqa: E402

            self._invocations = InvocationsClient(client_wrapper=self._client_wrapper)
        return self._invocations


class AsyncTestsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTestsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._folders: typing.Optional[AsyncFoldersClient] = None
        self._invocations: typing.Optional[AsyncInvocationsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawTestsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTestsClient
        """
        return self._raw_client

    async def create(
        self, *, request: TestsCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateAgentTestResponseModel:
        """
        Creates a new agent response test.

        Parameters
        ----------
        request : TestsCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateAgentTestResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs
        from elevenlabs.conversational_ai.tests import TestsCreateRequestBody_Llm

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.create(
                request=TestsCreateRequestBody_Llm(
                    name="name",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def move(
        self,
        *,
        entity_ids: typing.Sequence[str],
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Any:
        """
        Moves multiple tests or folders from one folder to another.

        Parameters
        ----------
        entity_ids : typing.Sequence[str]
            The IDs of tests or folders to move.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Tests or folders successfully moved to another folder

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.move(
                entity_ids=["entity_ids"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.move(entity_ids=entity_ids, move_to=move_to, request_options=request_options)
        return _response.data

    async def get(self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> TestsGetResponse:
        """
        Gets an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestsGetResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.get(
                test_id="TeaqRRdTcIfIu2i7BYfT",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(test_id, request_options=request_options)
        return _response.data

    async def update(
        self, test_id: str, *, request: TestsUpdateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> TestsUpdateResponse:
        """
        Updates an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request : TestsUpdateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestsUpdateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs
        from elevenlabs.conversational_ai.tests import TestsUpdateRequestBody_Llm

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.update(
                test_id="TeaqRRdTcIfIu2i7BYfT",
                request=TestsUpdateRequestBody_Llm(
                    name="name",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(test_id, request=request, request_options=request_options)
        return _response.data

    async def delete(self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Any:
        """
        Deletes an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.delete(
                test_id="TeaqRRdTcIfIu2i7BYfT",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(test_id, request_options=request_options)
        return _response.data

    async def summaries(
        self, *, test_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> GetTestsSummariesByIdsResponseModel:
        """
        Gets multiple agent response tests by their IDs. Returns a dictionary mapping test IDs to test summaries.

        Parameters
        ----------
        test_ids : typing.Sequence[str]
            List of test IDs to fetch. No duplicates allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTestsSummariesByIdsResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.summaries(
                test_ids=["test_id_1", "test_id_2"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.summaries(test_ids=test_ids, request_options=request_options)
        return _response.data

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        parent_folder_id: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]] = None,
        include_folders: typing.Optional[bool] = None,
        sort_mode: typing.Optional[TestsListRequestSortMode] = None,
        sharing_mode: typing.Optional[TestSharingMode] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetTestsPageResponseModel:
        """
        Lists all agent response tests with pagination support and optional search filtering.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many Tests to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            Search query to filter tests by name.

        parent_folder_id : typing.Optional[str]
            Filter by parent folder ID. Use 'root' to get items in the root folder.

        types : typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]]
            If present, the endpoint will return only tests/folders of the given types.

        include_folders : typing.Optional[bool]
            Deprecated. Use the `types` query param and include `folder` instead.

        sort_mode : typing.Optional[TestsListRequestSortMode]
            Sort mode for listing tests. Use 'folders_first' to place folders before tests.

        sharing_mode : typing.Optional[TestSharingMode]
            Filter test visibility. Use `shared_with_me` to return only tests/folders shared with the current user that they did not create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTestsPageResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.tests.list(
                cursor="cursor",
                page_size=1,
                search="search",
                parent_folder_id="parent_folder_id",
                types=["llm"],
                include_folders=True,
                sort_mode="default",
                sharing_mode="all",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            search=search,
            parent_folder_id=parent_folder_id,
            types=types,
            include_folders=include_folders,
            sort_mode=sort_mode,
            sharing_mode=sharing_mode,
            request_options=request_options,
        )
        return _response.data

    @property
    def folders(self):
        if self._folders is None:
            from .folders.client import AsyncFoldersClient  # noqa: E402

            self._folders = AsyncFoldersClient(client_wrapper=self._client_wrapper)
        return self._folders

    @property
    def invocations(self):
        if self._invocations is None:
            from .invocations.client import AsyncInvocationsClient  # noqa: E402

            self._invocations = AsyncInvocationsClient(client_wrapper=self._client_wrapper)
        return self._invocations



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .generation_config import GenerationConfig
from .pronunciation_dictionary_locator import PronunciationDictionaryLocator
from .realtime_voice_settings import RealtimeVoiceSettings


class InitializeConnection(UncheckedBaseModel):
    text: typing.Literal[" "] = pydantic.Field(default=" ")
    """
    The initial text that must be sent is a blank space.
    """

    voice_settings: typing.Optional[RealtimeVoiceSettings] = None
    generation_config: typing.Optional[GenerationConfig] = None
    pronunciation_dictionary_locators: typing.Optional[typing.List[PronunciationDictionaryLocator]] = pydantic.Field(
        default=None
    )
    """
    Optional list of pronunciation dictionary locators. If provided, these dictionaries will be used to
    modify pronunciation of matching text. Must only be provided in the first message.
    
    Note: Pronunciation dictionary matches will only be respected within a provided chunk.
    """

    xi_api_key: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="xi-api-key")] = pydantic.Field(
        default=None
    )
    """
    Your ElevenLabs API key. This can only be included in the first message and is not needed if present in the header.
    """

    authorization: typing.Optional[str] = pydantic.Field(default=None)
    """
    Your authorization bearer token. This can only be included in the first message and is not needed if present in the header.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.conv_ai_workspace_stored_secret_config import ConvAiWorkspaceStoredSecretConfig
from ...types.get_secret_dependencies_response_model import GetSecretDependenciesResponseModel
from ...types.get_workspace_secrets_response_model import GetWorkspaceSecretsResponseModel
from ...types.post_workspace_secret_response_model import PostWorkspaceSecretResponseModel
from ...types.secret_dependency_resource_type import SecretDependencyResourceType
from .raw_client import AsyncRawSecretsClient, RawSecretsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SecretsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSecretsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSecretsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSecretsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        dependency_limit: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetWorkspaceSecretsResponseModel:
        """
        Get all workspace secrets for the user

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100. If not provided, returns all secrets.

        dependency_limit : typing.Optional[int]
            Maximum number of dependent resources (tools, agents, phone numbers) to return per secret. Can not exceed 100.

        search : typing.Optional[str]
            If specified, returns only secrets whose names start with this string.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWorkspaceSecretsResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.secrets.list(
            page_size=1,
            dependency_limit=1,
            search="search",
            cursor="cursor",
        )
        """
        _response = self._raw_client.list(
            page_size=page_size,
            dependency_limit=dependency_limit,
            search=search,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    def create(
        self, *, name: str, value: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PostWorkspaceSecretResponseModel:
        """
        Create a new secret for the workspace

        Parameters
        ----------
        name : str

        value : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostWorkspaceSecretResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.secrets.create(
            name="name",
            value="value",
        )
        """
        _response = self._raw_client.create(name=name, value=value, request_options=request_options)
        return _response.data

    def get(
        self, secret_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConvAiWorkspaceStoredSecretConfig:
        """
        Get a workspace secret by ID

        Parameters
        ----------
        secret_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConvAiWorkspaceStoredSecretConfig
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.secrets.get(
            secret_id="secret_id",
        )
        """
        _response = self._raw_client.get(secret_id, request_options=request_options)
        return _response.data

    def delete(self, secret_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a workspace secret if it's not in use

        Parameters
        ----------
        secret_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.secrets.delete(
            secret_id="secret_id",
        )
        """
        _response = self._raw_client.delete(secret_id, request_options=request_options)
        return _response.data

    def update(
        self, secret_id: str, *, name: str, value: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PostWorkspaceSecretResponseModel:
        """
        Update an existing secret for the workspace

        Parameters
        ----------
        secret_id : str

        name : str

        value : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostWorkspaceSecretResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.secrets.update(
            secret_id="secret_id",
            name="name",
            value="value",
        )
        """
        _response = self._raw_client.update(secret_id, name=name, value=value, request_options=request_options)
        return _response.data

    def get_dependencies(
        self,
        secret_id: str,
        resource_type: SecretDependencyResourceType,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSecretDependenciesResponseModel:
        """
        Get paginated list of resources that depend on a specific secret, filtered by resource type.

        Parameters
        ----------
        secret_id : str

        resource_type : SecretDependencyResourceType

        page_size : typing.Optional[int]
            How many dependency items to return per page.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSecretDependenciesResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.secrets.get_dependencies(
            secret_id="secret_id",
            resource_type="tools",
            page_size=1,
            cursor="cursor",
        )
        """
        _response = self._raw_client.get_dependencies(
            secret_id, resource_type, page_size=page_size, cursor=cursor, request_options=request_options
        )
        return _response.data


class AsyncSecretsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSecretsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSecretsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSecretsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        dependency_limit: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetWorkspaceSecretsResponseModel:
        """
        Get all workspace secrets for the user

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100. If not provided, returns all secrets.

        dependency_limit : typing.Optional[int]
            Maximum number of dependent resources (tools, agents, phone numbers) to return per secret. Can not exceed 100.

        search : typing.Optional[str]
            If specified, returns only secrets whose names start with this string.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWorkspaceSecretsResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.secrets.list(
                page_size=1,
                dependency_limit=1,
                search="search",
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            page_size=page_size,
            dependency_limit=dependency_limit,
            search=search,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self, *, name: str, value: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PostWorkspaceSecretResponseModel:
        """
        Create a new secret for the workspace

        Parameters
        ----------
        name : str

        value : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostWorkspaceSecretResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.secrets.create(
                name="name",
                value="value",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(name=name, value=value, request_options=request_options)
        return _response.data

    async def get(
        self, secret_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConvAiWorkspaceStoredSecretConfig:
        """
        Get a workspace secret by ID

        Parameters
        ----------
        secret_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConvAiWorkspaceStoredSecretConfig
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.secrets.get(
                secret_id="secret_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(secret_id, request_options=request_options)
        return _response.data

    async def delete(self, secret_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a workspace secret if it's not in use

        Parameters
        ----------
        secret_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.secrets.delete(
                secret_id="secret_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(secret_id, request_options=request_options)
        return _response.data

    async def update(
        self, secret_id: str, *, name: str, value: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PostWorkspaceSecretResponseModel:
        """
        Update an existing secret for the workspace

        Parameters
        ----------
        secret_id : str

        name : str

        value : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostWorkspaceSecretResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.secrets.update(
                secret_id="secret_id",
                name="name",
                value="value",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(secret_id, name=name, value=value, request_options=request_options)
        return _response.data

    async def get_dependencies(
        self,
        secret_id: str,
        resource_type: SecretDependencyResourceType,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSecretDependenciesResponseModel:
        """
        Get paginated list of resources that depend on a specific secret, filtered by resource type.

        Parameters
        ----------
        secret_id : str

        resource_type : SecretDependencyResourceType

        page_size : typing.Optional[int]
            How many dependency items to return per page.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSecretDependenciesResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.secrets.get_dependencies(
                secret_id="secret_id",
                resource_type="tools",
                page_size=1,
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_dependencies(
            secret_id, resource_type, page_size=page_size, cursor=cursor, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..... import core
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.verify_pvc_voice_captcha_response_model import VerifyPvcVoiceCaptchaResponseModel
from .raw_client import AsyncRawCaptchaClient, RawCaptchaClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CaptchaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCaptchaClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCaptchaClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCaptchaClient
        """
        return self._raw_client

    def get(self, voice_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Get captcha for PVC voice verification.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.pvc.verification.captcha.get(
            voice_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.get(voice_id, request_options=request_options)
        return _response.data

    def verify(
        self, voice_id: str, *, recording: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> VerifyPvcVoiceCaptchaResponseModel:
        """
        Submit captcha verification for PVC voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        recording : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VerifyPvcVoiceCaptchaResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.pvc.verification.captcha.verify(
            voice_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.verify(voice_id, recording=recording, request_options=request_options)
        return _response.data


class AsyncCaptchaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCaptchaClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCaptchaClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCaptchaClient
        """
        return self._raw_client

    async def get(self, voice_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Get captcha for PVC voice verification.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.pvc.verification.captcha.get(
                voice_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(voice_id, request_options=request_options)
        return _response.data

    async def verify(
        self, voice_id: str, *, recording: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> VerifyPvcVoiceCaptchaResponseModel:
        """
        Submit captcha verification for PVC voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        recording : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VerifyPvcVoiceCaptchaResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.pvc.verification.captcha.verify(
                voice_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.verify(voice_id, recording=recording, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.get_conversation_users_page_response_model import GetConversationUsersPageResponseModel
from ...types.users_sort_by import UsersSortBy


class RawUsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        agent_id: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        call_start_before_unix: typing.Optional[int] = None,
        call_start_after_unix: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        sort_by: typing.Optional[UsersSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetConversationUsersPageResponseModel]:
        """
        Get distinct users from conversations with pagination.

        Parameters
        ----------
        agent_id : typing.Optional[str]
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        branch_id : typing.Optional[str]
            Filter conversations by branch ID.

        call_start_before_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations up to this start date.

        call_start_after_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations after to this start date.

        search : typing.Optional[str]
            Search/filter by user ID (exact match).

        page_size : typing.Optional[int]
            How many users to return at maximum. Defaults to 30.

        sort_by : typing.Optional[UsersSortBy]
            The field to sort the results by. Defaults to last_contact_unix_secs.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConversationUsersPageResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/users",
            method="GET",
            params={
                "agent_id": agent_id,
                "branch_id": branch_id,
                "call_start_before_unix": call_start_before_unix,
                "call_start_after_unix": call_start_after_unix,
                "search": search,
                "page_size": page_size,
                "sort_by": sort_by,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationUsersPageResponseModel,
                    construct_type(
                        type_=GetConversationUsersPageResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        agent_id: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        call_start_before_unix: typing.Optional[int] = None,
        call_start_after_unix: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        sort_by: typing.Optional[UsersSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetConversationUsersPageResponseModel]:
        """
        Get distinct users from conversations with pagination.

        Parameters
        ----------
        agent_id : typing.Optional[str]
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        branch_id : typing.Optional[str]
            Filter conversations by branch ID.

        call_start_before_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations up to this start date.

        call_start_after_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations after to this start date.

        search : typing.Optional[str]
            Search/filter by user ID (exact match).

        page_size : typing.Optional[int]
            How many users to return at maximum. Defaults to 30.

        sort_by : typing.Optional[UsersSortBy]
            The field to sort the results by. Defaults to last_contact_unix_secs.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConversationUsersPageResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/users",
            method="GET",
            params={
                "agent_id": agent_id,
                "branch_id": branch_id,
                "call_start_before_unix": call_start_before_unix,
                "call_start_after_unix": call_start_after_unix,
                "search": search,
                "page_size": page_size,
                "sort_by": sort_by,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationUsersPageResponseModel,
                    construct_type(
                        type_=GetConversationUsersPageResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class InputAudioChunkPayload(UncheckedBaseModel):
    """
    Payload for sending audio chunks from client to server.
    """

    message_type: typing.Literal["input_audio_chunk"] = pydantic.Field(default="input_audio_chunk")
    """
    The message type identifier.
    """

    audio_base_64: str = pydantic.Field()
    """
    Base64-encoded audio data.
    """

    commit: bool = pydantic.Field()
    """
    Whether to commit the transcription after this chunk.
    """

    sample_rate: int = pydantic.Field()
    """
    Sample rate of the audio in Hz.
    """

    previous_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Send text context to the model. Can only be sent alongside the first audio chunk. If sent in a subsequent chunk, an error will be returned.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .cue_options_request import CueOptionsRequest
from .media_id import MediaId


class OrderItemRequestOutput_Dub(UncheckedBaseModel):
    kind: typing.Literal["dub"] = "dub"
    media_id: MediaId
    source_language: str
    destination_languages: typing.List[str]
    include_captions: bool
    include_source_captions: bool
    instructions: typing.Optional[str] = None
    captions_sdh: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class OrderItemRequestOutput_Subtitles(UncheckedBaseModel):
    kind: typing.Literal["subtitles"] = "subtitles"
    media_ids: typing.List[MediaId]
    source_language: str
    destination_languages: typing.List[str]
    cue_options: typing.Optional[CueOptionsRequest] = None
    sdh: typing.Optional[bool] = None
    instructions: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


OrderItemRequestOutput = typing_extensions.Annotated[
    typing.Union[OrderItemRequestOutput_Dub, OrderItemRequestOutput_Subtitles], UnionMetadata(discriminant="kind")
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_voice_usage_model import ConversationVoiceUsageModel


class ConversationTtsUsageModel(UncheckedBaseModel):
    """
    Aggregated TTS usage for a conversation (analytics-only, not billing).
    """

    primary_tts_model: typing.Optional[str] = None
    total_audio_output_seconds: typing.Optional[float] = None
    total_characters: typing.Optional[int] = None
    per_voice_usage: typing.Optional[typing.List[ConversationVoiceUsageModel]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ContentThresholdGuardrailThreshold = typing.Union[
    float, typing.Literal["low"], typing.Literal["medium"], typing.Literal["high"]
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.composition_plan import CompositionPlan
from ....types.music_prompt import MusicPrompt

CompositionPlanCreateResponse = typing.Union[MusicPrompt, CompositionPlan]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_config_override_output import AgentConfigOverrideOutput
from .asr_conversational_config_override import AsrConversationalConfigOverride
from .conversation_config_override import ConversationConfigOverride
from .tts_conversational_config_override import TtsConversationalConfigOverride
from .turn_config_override import TurnConfigOverride


class ConversationConfigClientOverrideOutput(UncheckedBaseModel):
    asr: typing.Optional[AsrConversationalConfigOverride] = pydantic.Field(default=None)
    """
    Configuration for conversational transcription
    """

    turn: typing.Optional[TurnConfigOverride] = pydantic.Field(default=None)
    """
    Configuration for turn detection
    """

    tts: typing.Optional[TtsConversationalConfigOverride] = pydantic.Field(default=None)
    """
    Configuration for conversational text to speech
    """

    conversation: typing.Optional[ConversationConfigOverride] = pydantic.Field(default=None)
    """
    Configuration for conversational events
    """

    agent: typing.Optional[AgentConfigOverrideOutput] = pydantic.Field(default=None)
    """
    Agent specific configuration
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SpeakerSeparationResponseModelStatus = typing.Union[
    typing.Literal["not_started", "pending", "completed", "failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata


class ChapterContentBlockResponseModelNodesItem_TtsNode(UncheckedBaseModel):
    type: typing.Literal["tts_node"] = "tts_node"
    project_voice_ref_id: str
    text: str
    voice_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ChapterContentBlockResponseModelNodesItem_Other(UncheckedBaseModel):
    type: typing.Literal["_other"] = "_other"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ChapterContentBlockResponseModelNodesItem = typing_extensions.Annotated[
    typing.Union[ChapterContentBlockResponseModelNodesItem_TtsNode, ChapterContentBlockResponseModelNodesItem_Other],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawAudioClient, RawAudioClient


class AudioClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAudioClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAudioClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAudioClient
        """
        return self._raw_client

    def get(
        self, dubbing_id: str, language_code: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Returns dub as a streamed MP3 or MP4 file. If this dub has been edited using Dubbing Studio you need to use the resource render endpoint as this endpoint only returns the original automatic dub result.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language_code : str
            ID of the language.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            The dubbed audio or video file

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.audio.get(
            dubbing_id="dubbing_id",
            language_code="language_code",
        )
        """
        with self._raw_client.get(dubbing_id, language_code, request_options=request_options) as r:
            yield from r.data


class AsyncAudioClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAudioClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAudioClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAudioClient
        """
        return self._raw_client

    async def get(
        self, dubbing_id: str, language_code: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Returns dub as a streamed MP3 or MP4 file. If this dub has been edited using Dubbing Studio you need to use the resource render endpoint as this endpoint only returns the original automatic dub result.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language_code : str
            ID of the language.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            The dubbed audio or video file

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.audio.get(
                dubbing_id="dubbing_id",
                language_code="language_code",
            )


        asyncio.run(main())
        """
        async with self._raw_client.get(dubbing_id, language_code, request_options=request_options) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.single_use_token_response_model import SingleUseTokenResponseModel
from ...types.single_use_token_type import SingleUseTokenType
from .raw_client import AsyncRawSingleUseClient, RawSingleUseClient


class SingleUseClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSingleUseClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSingleUseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSingleUseClient
        """
        return self._raw_client

    def create(
        self, token_type: SingleUseTokenType, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SingleUseTokenResponseModel:
        """
        Generate a time limited single-use token with embedded authentication for frontend clients.

        Parameters
        ----------
        token_type : SingleUseTokenType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SingleUseTokenResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.tokens.single_use.create(
            token_type="realtime_scribe",
        )
        """
        _response = self._raw_client.create(token_type, request_options=request_options)
        return _response.data


class AsyncSingleUseClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSingleUseClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSingleUseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSingleUseClient
        """
        return self._raw_client

    async def create(
        self, token_type: SingleUseTokenType, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SingleUseTokenResponseModel:
        """
        Generate a time limited single-use token with embedded authentication for frontend clients.

        Parameters
        ----------
        token_type : SingleUseTokenType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SingleUseTokenResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tokens.single_use.create(
                token_type="realtime_scribe",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(token_type, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .composition_plan import CompositionPlan
from .music_prompt import MusicPrompt

DetailedMusicResponseCompositionPlan = typing.Union[MusicPrompt, CompositionPlan]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .body_compose_music_v_1_music_post_composition_plan import BodyComposeMusicV1MusicPostCompositionPlan
    from .body_compose_music_v_1_music_post_model_id import BodyComposeMusicV1MusicPostModelId
    from .body_compose_music_with_a_detailed_response_v_1_music_detailed_post_composition_plan import (
        BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan,
    )
    from .body_compose_music_with_a_detailed_response_v_1_music_detailed_post_model_id import (
        BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId,
    )
    from .body_stream_composed_music_v_1_music_stream_post_composition_plan import (
        BodyStreamComposedMusicV1MusicStreamPostCompositionPlan,
    )
    from .body_stream_composed_music_v_1_music_stream_post_model_id import (
        BodyStreamComposedMusicV1MusicStreamPostModelId,
    )
    from .music_compose_detailed_request_output_format import MusicComposeDetailedRequestOutputFormat
    from .music_compose_request_output_format import MusicComposeRequestOutputFormat
    from .music_separate_stems_request_stem_variation_id import MusicSeparateStemsRequestStemVariationId
    from .music_stream_request_output_format import MusicStreamRequestOutputFormat
    from .music_video_to_music_request_model_id import MusicVideoToMusicRequestModelId
_dynamic_imports: typing.Dict[str, str] = {
    "BodyComposeMusicV1MusicPostCompositionPlan": ".body_compose_music_v_1_music_post_composition_plan",
    "BodyComposeMusicV1MusicPostModelId": ".body_compose_music_v_1_music_post_model_id",
    "BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan": ".body_compose_music_with_a_detailed_response_v_1_music_detailed_post_composition_plan",
    "BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId": ".body_compose_music_with_a_detailed_response_v_1_music_detailed_post_model_id",
    "BodyStreamComposedMusicV1MusicStreamPostCompositionPlan": ".body_stream_composed_music_v_1_music_stream_post_composition_plan",
    "BodyStreamComposedMusicV1MusicStreamPostModelId": ".body_stream_composed_music_v_1_music_stream_post_model_id",
    "MusicComposeDetailedRequestOutputFormat": ".music_compose_detailed_request_output_format",
    "MusicComposeRequestOutputFormat": ".music_compose_request_output_format",
    "MusicSeparateStemsRequestStemVariationId": ".music_separate_stems_request_stem_variation_id",
    "MusicStreamRequestOutputFormat": ".music_stream_request_output_format",
    "MusicVideoToMusicRequestModelId": ".music_video_to_music_request_model_id",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BodyComposeMusicV1MusicPostCompositionPlan",
    "BodyComposeMusicV1MusicPostModelId",
    "BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan",
    "BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId",
    "BodyStreamComposedMusicV1MusicStreamPostCompositionPlan",
    "BodyStreamComposedMusicV1MusicStreamPostModelId",
    "MusicComposeDetailedRequestOutputFormat",
    "MusicComposeRequestOutputFormat",
    "MusicSeparateStemsRequestStemVariationId",
    "MusicStreamRequestOutputFormat",
    "MusicVideoToMusicRequestModelId",
]



# This file was auto-generated by Fern from our API Definition.

import typing

WorkspaceGroupPermission = typing.Union[
    typing.Literal[
        "text_to_speech",
        "speech_to_speech",
        "speech_to_text",
        "voice_lab",
        "sound_effects",
        "projects",
        "voiceover_studio",
        "dubbing",
        "audio_native",
        "conversational_ai",
        "conversational_ai_read",
        "voice_isolator",
        "ai_speech_classifier",
        "add_voice_from_voice_library",
        "create_instant_voice_clone",
        "create_professional_voice_clone",
        "create_user_api_key",
        "publish_studio_project",
        "music",
        "image_video_generation",
        "share_voice_externally",
        "publish_voice_to_voice_library",
        "view_fiat_balance",
        "workspace_analytics_full_read",
        "service_accounts_manage",
        "webhooks_manage",
        "group_members_manage",
        "workspace_members_invite",
        "workspace_members_remove",
        "terms_of_service_accept",
        "audit_log_read",
        "copy_resources_cross_workspace",
        "voice_design",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.add_pronunciation_dictionary_response_model import AddPronunciationDictionaryResponseModel
from ..types.get_pronunciation_dictionaries_metadata_response_model import (
    GetPronunciationDictionariesMetadataResponseModel,
)
from ..types.get_pronunciation_dictionary_metadata_response import GetPronunciationDictionaryMetadataResponse
from ..types.get_pronunciation_dictionary_with_rules_response_model import (
    GetPronunciationDictionaryWithRulesResponseModel,
)
from .raw_client import AsyncRawPronunciationDictionariesClient, RawPronunciationDictionariesClient
from .types.body_add_a_pronunciation_dictionary_v_1_pronunciation_dictionaries_add_from_rules_post_rules_item import (
    BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem,
)
from .types.body_add_a_pronunciation_dictionary_v_1_pronunciation_dictionaries_add_from_rules_post_workspace_access import (
    BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostWorkspaceAccess,
)
from .types.pronunciation_dictionaries_create_from_file_request_workspace_access import (
    PronunciationDictionariesCreateFromFileRequestWorkspaceAccess,
)
from .types.pronunciation_dictionaries_list_request_sort import PronunciationDictionariesListRequestSort

if typing.TYPE_CHECKING:
    from .rules.client import AsyncRulesClient, RulesClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PronunciationDictionariesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPronunciationDictionariesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._rules: typing.Optional[RulesClient] = None

    @property
    def with_raw_response(self) -> RawPronunciationDictionariesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPronunciationDictionariesClient
        """
        return self._raw_client

    def create_from_file(
        self,
        *,
        name: str,
        file: typing.Optional[core.File] = OMIT,
        description: typing.Optional[str] = OMIT,
        workspace_access: typing.Optional[PronunciationDictionariesCreateFromFileRequestWorkspaceAccess] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddPronunciationDictionaryResponseModel:
        """
        Creates a new pronunciation dictionary from a lexicon .PLS file

        Parameters
        ----------
        name : str
            The name of the pronunciation dictionary, used for identification only.

        file : typing.Optional[core.File]
            See core.File for more documentation

        description : typing.Optional[str]
            A description of the pronunciation dictionary, used for identification only.

        workspace_access : typing.Optional[PronunciationDictionariesCreateFromFileRequestWorkspaceAccess]
            Should be one of 'admin', 'editor' or 'viewer'. If not provided, defaults to no access.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddPronunciationDictionaryResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.pronunciation_dictionaries.create_from_file(
            name="name",
        )
        """
        _response = self._raw_client.create_from_file(
            name=name,
            file=file,
            description=description,
            workspace_access=workspace_access,
            request_options=request_options,
        )
        return _response.data

    def create_from_rules(
        self,
        *,
        rules: typing.Sequence[BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem],
        name: str,
        description: typing.Optional[str] = OMIT,
        workspace_access: typing.Optional[
            BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostWorkspaceAccess
        ] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddPronunciationDictionaryResponseModel:
        """
        Creates a new pronunciation dictionary from provided rules.

        Parameters
        ----------
        rules : typing.Sequence[BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem]
            List of pronunciation rules. Rule can be either:
                an alias rule: {'string_to_replace': 'a', 'type': 'alias', 'alias': 'b', }
                or a phoneme rule: {'string_to_replace': 'a', 'type': 'phoneme', 'phoneme': 'b', 'alphabet': 'ipa' }

        name : str
            The name of the pronunciation dictionary, used for identification only.

        description : typing.Optional[str]
            A description of the pronunciation dictionary, used for identification only.

        workspace_access : typing.Optional[BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostWorkspaceAccess]
            Should be one of 'admin', 'editor' or 'viewer'. If not provided, defaults to no access.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddPronunciationDictionaryResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs
        from elevenlabs.pronunciation_dictionaries import (
            BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem_Alias,
        )

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.pronunciation_dictionaries.create_from_rules(
            rules=[
                BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem_Alias(
                    string_to_replace="Thailand",
                    case_sensitive=True,
                    word_boundaries=True,
                    alias="tie-land",
                )
            ],
            name="My Dictionary",
        )
        """
        _response = self._raw_client.create_from_rules(
            rules=rules,
            name=name,
            description=description,
            workspace_access=workspace_access,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, pronunciation_dictionary_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPronunciationDictionaryWithRulesResponseModel:
        """
        Get metadata for a pronunciation dictionary

        Parameters
        ----------
        pronunciation_dictionary_id : str
            The id of the pronunciation dictionary

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPronunciationDictionaryWithRulesResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.pronunciation_dictionaries.get(
            pronunciation_dictionary_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.get(pronunciation_dictionary_id, request_options=request_options)
        return _response.data

    def update(
        self,
        pronunciation_dictionary_id: str,
        *,
        archived: typing.Optional[bool] = OMIT,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetPronunciationDictionaryMetadataResponse:
        """
        Partially update the pronunciation dictionary without changing the version

        Parameters
        ----------
        pronunciation_dictionary_id : str
            The id of the pronunciation dictionary

        archived : typing.Optional[bool]
            Whether to archive the pronunciation dictionary.

        name : typing.Optional[str]
            The name of the pronunciation dictionary, used for identification only.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPronunciationDictionaryMetadataResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.pronunciation_dictionaries.update(
            pronunciation_dictionary_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.update(
            pronunciation_dictionary_id, archived=archived, name=name, request_options=request_options
        )
        return _response.data

    def download(
        self, dictionary_id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Get a PLS file with a pronunciation dictionary version rules

        Parameters
        ----------
        dictionary_id : str
            The id of the pronunciation dictionary

        version_id : str
            The id of the pronunciation dictionary version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            The PLS file containing pronunciation dictionary rules

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.pronunciation_dictionaries.download(
            dictionary_id="dictionary_id",
            version_id="version_id",
        )
        """
        with self._raw_client.download(dictionary_id, version_id, request_options=request_options) as r:
            yield from r.data

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        sort: typing.Optional[PronunciationDictionariesListRequestSort] = None,
        sort_direction: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetPronunciationDictionariesMetadataResponseModel:
        """
        Get a list of the pronunciation dictionaries you have access to and their metadata

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many pronunciation dictionaries to return at maximum. Can not exceed 100, defaults to 30.

        sort : typing.Optional[PronunciationDictionariesListRequestSort]
            Which field to sort by, one of 'created_at_unix' or 'name'.

        sort_direction : typing.Optional[str]
            Which direction to sort the voices in. 'ascending' or 'descending'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPronunciationDictionariesMetadataResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.pronunciation_dictionaries.list(
            cursor="cursor",
            page_size=1,
            sort="creation_time_unix",
            sort_direction="sort_direction",
        )
        """
        _response = self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            sort=sort,
            sort_direction=sort_direction,
            request_options=request_options,
        )
        return _response.data

    @property
    def rules(self):
        if self._rules is None:
            from .rules.client import RulesClient  # noqa: E402

            self._rules = RulesClient(client_wrapper=self._client_wrapper)
        return self._rules


class AsyncPronunciationDictionariesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPronunciationDictionariesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._rules: typing.Optional[AsyncRulesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawPronunciationDictionariesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPronunciationDictionariesClient
        """
        return self._raw_client

    async def create_from_file(
        self,
        *,
        name: str,
        file: typing.Optional[core.File] = OMIT,
        description: typing.Optional[str] = OMIT,
        workspace_access: typing.Optional[PronunciationDictionariesCreateFromFileRequestWorkspaceAccess] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddPronunciationDictionaryResponseModel:
        """
        Creates a new pronunciation dictionary from a lexicon .PLS file

        Parameters
        ----------
        name : str
            The name of the pronunciation dictionary, used for identification only.

        file : typing.Optional[core.File]
            See core.File for more documentation

        description : typing.Optional[str]
            A description of the pronunciation dictionary, used for identification only.

        workspace_access : typing.Optional[PronunciationDictionariesCreateFromFileRequestWorkspaceAccess]
            Should be one of 'admin', 'editor' or 'viewer'. If not provided, defaults to no access.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddPronunciationDictionaryResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pronunciation_dictionaries.create_from_file(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_file(
            name=name,
            file=file,
            description=description,
            workspace_access=workspace_access,
            request_options=request_options,
        )
        return _response.data

    async def create_from_rules(
        self,
        *,
        rules: typing.Sequence[BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem],
        name: str,
        description: typing.Optional[str] = OMIT,
        workspace_access: typing.Optional[
            BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostWorkspaceAccess
        ] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddPronunciationDictionaryResponseModel:
        """
        Creates a new pronunciation dictionary from provided rules.

        Parameters
        ----------
        rules : typing.Sequence[BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem]
            List of pronunciation rules. Rule can be either:
                an alias rule: {'string_to_replace': 'a', 'type': 'alias', 'alias': 'b', }
                or a phoneme rule: {'string_to_replace': 'a', 'type': 'phoneme', 'phoneme': 'b', 'alphabet': 'ipa' }

        name : str
            The name of the pronunciation dictionary, used for identification only.

        description : typing.Optional[str]
            A description of the pronunciation dictionary, used for identification only.

        workspace_access : typing.Optional[BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostWorkspaceAccess]
            Should be one of 'admin', 'editor' or 'viewer'. If not provided, defaults to no access.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddPronunciationDictionaryResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs
        from elevenlabs.pronunciation_dictionaries import (
            BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem_Alias,
        )

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pronunciation_dictionaries.create_from_rules(
                rules=[
                    BodyAddAPronunciationDictionaryV1PronunciationDictionariesAddFromRulesPostRulesItem_Alias(
                        string_to_replace="Thailand",
                        case_sensitive=True,
                        word_boundaries=True,
                        alias="tie-land",
                    )
                ],
                name="My Dictionary",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_rules(
            rules=rules,
            name=name,
            description=description,
            workspace_access=workspace_access,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, pronunciation_dictionary_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPronunciationDictionaryWithRulesResponseModel:
        """
        Get metadata for a pronunciation dictionary

        Parameters
        ----------
        pronunciation_dictionary_id : str
            The id of the pronunciation dictionary

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPronunciationDictionaryWithRulesResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pronunciation_dictionaries.get(
                pronunciation_dictionary_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(pronunciation_dictionary_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        pronunciation_dictionary_id: str,
        *,
        archived: typing.Optional[bool] = OMIT,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetPronunciationDictionaryMetadataResponse:
        """
        Partially update the pronunciation dictionary without changing the version

        Parameters
        ----------
        pronunciation_dictionary_id : str
            The id of the pronunciation dictionary

        archived : typing.Optional[bool]
            Whether to archive the pronunciation dictionary.

        name : typing.Optional[str]
            The name of the pronunciation dictionary, used for identification only.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPronunciationDictionaryMetadataResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pronunciation_dictionaries.update(
                pronunciation_dictionary_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            pronunciation_dictionary_id, archived=archived, name=name, request_options=request_options
        )
        return _response.data

    async def download(
        self, dictionary_id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Get a PLS file with a pronunciation dictionary version rules

        Parameters
        ----------
        dictionary_id : str
            The id of the pronunciation dictionary

        version_id : str
            The id of the pronunciation dictionary version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            The PLS file containing pronunciation dictionary rules

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pronunciation_dictionaries.download(
                dictionary_id="dictionary_id",
                version_id="version_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.download(dictionary_id, version_id, request_options=request_options) as r:
            async for _chunk in r.data:
                yield _chunk

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        sort: typing.Optional[PronunciationDictionariesListRequestSort] = None,
        sort_direction: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetPronunciationDictionariesMetadataResponseModel:
        """
        Get a list of the pronunciation dictionaries you have access to and their metadata

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many pronunciation dictionaries to return at maximum. Can not exceed 100, defaults to 30.

        sort : typing.Optional[PronunciationDictionariesListRequestSort]
            Which field to sort by, one of 'created_at_unix' or 'name'.

        sort_direction : typing.Optional[str]
            Which direction to sort the voices in. 'ascending' or 'descending'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPronunciationDictionariesMetadataResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pronunciation_dictionaries.list(
                cursor="cursor",
                page_size=1,
                sort="creation_time_unix",
                sort_direction="sort_direction",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            sort=sort,
            sort_direction=sort_direction,
            request_options=request_options,
        )
        return _response.data

    @property
    def rules(self):
        if self._rules is None:
            from .rules.client import AsyncRulesClient  # noqa: E402

            self._rules = AsyncRulesClient(client_wrapper=self._client_wrapper)
        return self._rules



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        McpServerConfigUpdateRequestModelAuthConnection,
        McpServerConfigUpdateRequestModelRequestHeadersValue,
    )
    from . import approval_policy, tool_approvals, tool_configs, tools
    from .tool_configs import (
        McpToolConfigOverrideCreateRequestModelInputOverridesValue,
        McpToolConfigOverrideCreateRequestModelInputOverridesValue_Constant,
        McpToolConfigOverrideCreateRequestModelInputOverridesValue_DynamicVariable,
        McpToolConfigOverrideCreateRequestModelInputOverridesValue_Llm,
        McpToolConfigOverrideCreateRequestModelInputOverridesValue_Omit,
        McpToolConfigOverrideUpdateRequestModelInputOverridesValue,
        McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Constant,
        McpToolConfigOverrideUpdateRequestModelInputOverridesValue_DynamicVariable,
        McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Llm,
        McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Omit,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "McpServerConfigUpdateRequestModelAuthConnection": ".types",
    "McpServerConfigUpdateRequestModelRequestHeadersValue": ".types",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue": ".tool_configs",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_Constant": ".tool_configs",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_DynamicVariable": ".tool_configs",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_Llm": ".tool_configs",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_Omit": ".tool_configs",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue": ".tool_configs",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Constant": ".tool_configs",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_DynamicVariable": ".tool_configs",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Llm": ".tool_configs",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Omit": ".tool_configs",
    "approval_policy": ".approval_policy",
    "tool_approvals": ".tool_approvals",
    "tool_configs": ".tool_configs",
    "tools": ".tools",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "McpServerConfigUpdateRequestModelAuthConnection",
    "McpServerConfigUpdateRequestModelRequestHeadersValue",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_Constant",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_DynamicVariable",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_Llm",
    "McpToolConfigOverrideCreateRequestModelInputOverridesValue_Omit",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Constant",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_DynamicVariable",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Llm",
    "McpToolConfigOverrideUpdateRequestModelInputOverridesValue_Omit",
    "approval_policy",
    "tool_approvals",
    "tool_configs",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .url_model import UrlModel


class HttpRequestModel(UncheckedBaseModel):
    """
    HTTP request details.

    Spec: https://schema.ocsf.io/1.6.0/objects/http_request
    """

    http_method: str = pydantic.Field()
    """
    HTTP method (GET, POST, etc.)
    """

    url: UrlModel = pydantic.Field()
    """
    Request URL object
    """

    user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    User agent string
    """

    x_forwarded_for: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    X-Forwarded-For header as a list
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.conv_ai_dynamic_variable import ConvAiDynamicVariable
from ....types.conv_ai_env_var_locator import ConvAiEnvVarLocator
from ....types.conv_ai_secret_locator import ConvAiSecretLocator

McpServerConfigUpdateRequestModelRequestHeadersValue = typing.Union[
    str, ConvAiSecretLocator, ConvAiDynamicVariable, ConvAiEnvVarLocator
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .open_aimcp_tool_require_approval import OpenAimcpToolRequireApproval


class OpenAiSessionConfigToolsItem_Function(UncheckedBaseModel):
    type: typing.Literal["function"] = "function"
    name: str
    description: typing.Optional[str] = None
    parameters: typing.Optional[typing.Dict[str, typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class OpenAiSessionConfigToolsItem_Mcp(UncheckedBaseModel):
    type: typing.Literal["mcp"] = "mcp"
    server_label: str
    server_url: str
    authorization: typing.Optional[str] = None
    allowed_tools: typing.Optional[typing.List[str]] = None
    require_approval: typing.Optional[OpenAimcpToolRequireApproval] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


OpenAiSessionConfigToolsItem = typing_extensions.Annotated[
    typing.Union[OpenAiSessionConfigToolsItem_Function, OpenAiSessionConfigToolsItem_Mcp],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.workspace_group_by_name_response_model import WorkspaceGroupByNameResponseModel
from ...types.workspace_group_response_model import WorkspaceGroupResponseModel
from .raw_client import AsyncRawGroupsClient, RawGroupsClient

if typing.TYPE_CHECKING:
    from .members.client import AsyncMembersClient, MembersClient


class GroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGroupsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._members: typing.Optional[MembersClient] = None

    @property
    def with_raw_response(self) -> RawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGroupsClient
        """
        return self._raw_client

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, WorkspaceGroupResponseModel]:
        """
        Get all groups in the workspace

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, WorkspaceGroupResponseModel]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.groups.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def search(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[WorkspaceGroupByNameResponseModel]:
        """
        Searches for user groups in the workspace. Multiple or no groups may be returned.

        Parameters
        ----------
        name : str
            Name of the target group.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[WorkspaceGroupByNameResponseModel]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.groups.search(
            name="name",
        )
        """
        _response = self._raw_client.search(name=name, request_options=request_options)
        return _response.data

    @property
    def members(self):
        if self._members is None:
            from .members.client import MembersClient  # noqa: E402

            self._members = MembersClient(client_wrapper=self._client_wrapper)
        return self._members


class AsyncGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGroupsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._members: typing.Optional[AsyncMembersClient] = None

    @property
    def with_raw_response(self) -> AsyncRawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGroupsClient
        """
        return self._raw_client

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, WorkspaceGroupResponseModel]:
        """
        Get all groups in the workspace

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, WorkspaceGroupResponseModel]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.groups.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def search(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[WorkspaceGroupByNameResponseModel]:
        """
        Searches for user groups in the workspace. Multiple or no groups may be returned.

        Parameters
        ----------
        name : str
            Name of the target group.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[WorkspaceGroupByNameResponseModel]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.groups.search(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search(name=name, request_options=request_options)
        return _response.data

    @property
    def members(self):
        if self._members is None:
            from .members.client import AsyncMembersClient  # noqa: E402

            self._members = AsyncMembersClient(client_wrapper=self._client_wrapper)
        return self._members



import asyncio
import json
import subprocess
import typing
from enum import Enum

if typing.TYPE_CHECKING:
    from websockets.asyncio.client import ClientConnection


class RealtimeEvents(str, Enum):
    """Events emitted by the RealtimeConnection"""
    OPEN = "open"
    CLOSE = "close"
    SESSION_STARTED = "session_started"
    PARTIAL_TRANSCRIPT = "partial_transcript"
    COMMITTED_TRANSCRIPT = "committed_transcript"
    COMMITTED_TRANSCRIPT_WITH_TIMESTAMPS = "committed_transcript_with_timestamps"
    ERROR = "error"
    AUTH_ERROR = "auth_error"
    QUOTA_EXCEEDED = "quota_exceeded"
    COMMIT_THROTTLED = "commit_throttled"
    TRANSCRIBER_ERROR = "transcriber_error"
    UNACCEPTED_TERMS_ERROR = "unaccepted_terms_error"
    RATE_LIMITED = "rate_limited"
    INPUT_ERROR = "input_error"
    QUEUE_OVERFLOW = "queue_overflow"
    RESOURCE_EXHAUSTED = "resource_exhausted"
    SESSION_TIME_LIMIT_EXCEEDED = "session_time_limit_exceeded"
    CHUNK_SIZE_EXCEEDED = "chunk_size_exceeded"
    INSUFFICIENT_AUDIO_ACTIVITY = "insufficient_audio_activity"



class RealtimeConnection:
    """
    A WebSocket connection for real-time speech-to-text transcription.

    This class handles bidirectional WebSocket communication with the ElevenLabs
    speech-to-text API, managing audio streaming and receiving transcription results.

    Example:
        ```python
        connection = await client.speech_to_text.realtime.connect({
            "audio_format": AudioFormat.PCM_16000,
            "sample_rate": 16000
        })

        connection.on(RealtimeEvents.PARTIAL_TRANSCRIPT, lambda data: print(data))
        connection.on(RealtimeEvents.COMMITTED_TRANSCRIPT, lambda data: print(data))

        # Send audio
        connection.send({"audioBase64": audio_chunk})

        # When done
        connection.commit()
        await connection.close()
        ```
    """

    def __init__(self, websocket: "ClientConnection", current_sample_rate: int, ffmpeg_process: typing.Optional[subprocess.Popen] = None):
        self.websocket = websocket
        self.current_sample_rate = current_sample_rate
        self.ffmpeg_process = ffmpeg_process
        self._event_handlers: typing.Dict[str, typing.List[typing.Callable]] = {}
        self._message_task: typing.Optional[asyncio.Task] = None

    def on(self, event: str, callback: typing.Callable) -> None:
        """
        Register an event handler for a specific event type.

        Args:
            event: The event type to listen for (from RealtimeEvents enum)
            callback: The function to call when the event occurs

        Example:
            ```python
            def handle_transcript(data):
                print(f"Transcript: {data['transcript']}")

            connection.on(RealtimeEvents.PARTIAL_TRANSCRIPT, handle_transcript)
            ```
        """
        if event not in self._event_handlers:
            self._event_handlers[event] = []
        self._event_handlers[event].append(callback)

    def _emit(self, event: str, *args) -> None:
        """Emit an event to all registered handlers"""
        if event in self._event_handlers:
            for handler in self._event_handlers[event]:
                try:
                    handler(*args)
                except Exception as e:
                    print(f"Error in event handler for {event}: {e}")

    async def _start_message_handler(self) -> None:
        """Start handling incoming WebSocket messages"""
        try:
            async for message in self.websocket:
                try:
                    data = json.loads(message)
                    message_type = data.get("message_type")

                    # Try to match message_type to a known event
                    try:
                        event = RealtimeEvents(message_type)
                        self._emit(event, data)

                        # Also emit generic ERROR event for specific error types
                        error_events = {
                            RealtimeEvents.AUTH_ERROR,
                            RealtimeEvents.QUOTA_EXCEEDED,
                            RealtimeEvents.COMMIT_THROTTLED,
                            RealtimeEvents.TRANSCRIBER_ERROR,
                            RealtimeEvents.UNACCEPTED_TERMS_ERROR,
                            RealtimeEvents.RATE_LIMITED,
                            RealtimeEvents.INPUT_ERROR,
                            RealtimeEvents.QUEUE_OVERFLOW,
                            RealtimeEvents.RESOURCE_EXHAUSTED,
                            RealtimeEvents.SESSION_TIME_LIMIT_EXCEEDED,
                            RealtimeEvents.CHUNK_SIZE_EXCEEDED,
                            RealtimeEvents.INSUFFICIENT_AUDIO_ACTIVITY,
                        }
                        if event in error_events:
                            self._emit(RealtimeEvents.ERROR, data)
                    except ValueError:
                        # Unknown message type, ignore
                        pass
                except json.JSONDecodeError as e:
                    self._emit(RealtimeEvents.ERROR, {"error": f"Failed to parse message: {e}"})
        except Exception as e:
            self._emit(RealtimeEvents.ERROR, {"error": str(e)})
        finally:
            self._emit(RealtimeEvents.CLOSE)

    async def send(self, data: typing.Dict[str, typing.Any]) -> None:
        """
        Send an audio chunk to the server for transcription.

        Args:
            data: Dictionary containing the following keys:
                - audio_base_64 (str): Base64-encoded audio data to transcribe
                - previous_text (str, optional): Previous transcript text to provide context
                  for more accurate transcription

        Raises:
            RuntimeError: If the WebSocket connection is not open

        Example:
            ```python
            # Send audio chunk
            connection.send({
                "audio_base_64": base64_encoded_audio
            })

            # Send audio chunk with context - can only be sent with the first chunk of audio
            connection.send({
                "audio_base_64": base64_encoded_audio,
                "previous_text": "Previously transcribed text for context"
            })
            ```
        """
        if not self.websocket:
            raise RuntimeError("WebSocket is not connected")

        message = {
            "message_type": "input_audio_chunk",
            "audio_base_64": data.get("audio_base_64", ""),
            "commit": False,
            "sample_rate": self.current_sample_rate,
            "previous_text": data.get("previous_text"),
        }

        await self.websocket.send(json.dumps(message))

    async def commit(self) -> None:
        """
        Commits the segment, triggering a COMMITTED_TRANSCRIPT event and clearing the buffer.
        It's recommend to commit often when using CommitStrategy.MANUAL to keep latency low.

        Raises:
            RuntimeError: If the WebSocket connection is not open

        Remarks:
            Only needed when using CommitStrategy.MANUAL.
            When using CommitStrategy.VAD, commits are handled automatically by the server.

        Example:
            ```python
            # Send all audio chunks
            for chunk in audio_chunks:
                connection.send({"audioBase64": chunk})

            # Commit the audio segment
            await connection.commit()
            ```
        """
        if not self.websocket:
            raise RuntimeError("WebSocket is not connected")

        message = {
            "message_type": "input_audio_chunk",
            "audio_base_64": "",
            "commit": True,
            "sample_rate": self.current_sample_rate,
        }

        await self.websocket.send(json.dumps(message))

    async def close(self) -> None:
        """
        Closes the WebSocket connection and cleans up resources.
        This will terminate any ongoing transcription and stop ffmpeg processes if running.

        Remarks:
            After calling close(), this connection cannot be reused.
            Create a new connection if you need to start transcribing again.

        Example:
            ```python
            connection.on(RealtimeEvents.COMMITTED_TRANSCRIPT, async lambda data: (
                print("Committed:", data["transcript"]),
                await connection.close()
            ))
            ```
        """
        await self._cleanup()
        if self.websocket:
            await self.websocket.close(1000, "User ended conversation")
        if self._message_task and not self._message_task.done():
            self._message_task.cancel()
            try:
                await self._message_task
            except asyncio.CancelledError:
                pass

    async def _cleanup(self) -> None:
        """Clean up resources like ffmpeg processes"""
        if self.ffmpeg_process:
            self.ffmpeg_process.kill()
            try:
                self.ffmpeg_process.wait(timeout=1)
            except subprocess.TimeoutExpired:
                self.ffmpeg_process.kill()
            self.ffmpeg_process = None




# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.environment_variable_response import EnvironmentVariableResponse
from ..types.environment_variables_list_response import EnvironmentVariablesListResponse
from .raw_client import AsyncRawEnvironmentVariablesClient, RawEnvironmentVariablesClient
from .types.environment_variables_create_request_body import EnvironmentVariablesCreateRequestBody
from .types.environment_variables_list_request_type import EnvironmentVariablesListRequestType
from .types.update_environment_variable_request_values_value import UpdateEnvironmentVariableRequestValuesValue

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EnvironmentVariablesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEnvironmentVariablesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEnvironmentVariablesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEnvironmentVariablesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        label: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        type: typing.Optional[EnvironmentVariablesListRequestType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EnvironmentVariablesListResponse:
        """
        List all environment variables for the workspace with optional filtering

        Parameters
        ----------
        cursor : typing.Optional[str]
            Pagination cursor from previous response

        page_size : typing.Optional[int]
            Number of items to return (1-100)

        label : typing.Optional[str]
            Filter by exact label match

        environment : typing.Optional[str]
            Filter to only return variables that have this environment. When specified, the values dict in the response will only contain this environment.

        type : typing.Optional[EnvironmentVariablesListRequestType]
            Filter by variable type

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariablesListResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.environment_variables.list(
            cursor="cursor",
            page_size=1,
            label="label",
            environment="environment",
            type="string",
        )
        """
        _response = self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            label=label,
            environment=environment,
            type=type,
            request_options=request_options,
        )
        return _response.data

    def create(
        self, *, request: EnvironmentVariablesCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> EnvironmentVariableResponse:
        """
        Create a new environment variable for the workspace

        Parameters
        ----------
        request : EnvironmentVariablesCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariableResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs
        from elevenlabs.environment_variables import (
            EnvironmentVariablesCreateRequestBody_String,
        )

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.environment_variables.create(
            request=EnvironmentVariablesCreateRequestBody_String(
                label="label",
                values={"key": "value"},
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def get(
        self, env_var_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EnvironmentVariableResponse:
        """
        Get a specific environment variable by ID

        Parameters
        ----------
        env_var_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariableResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.environment_variables.get(
            env_var_id="env_var_id",
        )
        """
        _response = self._raw_client.get(env_var_id, request_options=request_options)
        return _response.data

    def update(
        self,
        env_var_id: str,
        *,
        values: typing.Dict[str, typing.Optional[UpdateEnvironmentVariableRequestValuesValue]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EnvironmentVariableResponse:
        """
        Replace an environment variable's values. Use null to remove an environment (except production).

        Parameters
        ----------
        env_var_id : str

        values : typing.Dict[str, typing.Optional[UpdateEnvironmentVariableRequestValuesValue]]
            Values to replace. Set to null to remove an environment (except 'production').

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariableResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.environment_variables.update(
            env_var_id="env_var_id",
            values={},
        )
        """
        _response = self._raw_client.update(env_var_id, values=values, request_options=request_options)
        return _response.data


class AsyncEnvironmentVariablesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEnvironmentVariablesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEnvironmentVariablesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEnvironmentVariablesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        label: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        type: typing.Optional[EnvironmentVariablesListRequestType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EnvironmentVariablesListResponse:
        """
        List all environment variables for the workspace with optional filtering

        Parameters
        ----------
        cursor : typing.Optional[str]
            Pagination cursor from previous response

        page_size : typing.Optional[int]
            Number of items to return (1-100)

        label : typing.Optional[str]
            Filter by exact label match

        environment : typing.Optional[str]
            Filter to only return variables that have this environment. When specified, the values dict in the response will only contain this environment.

        type : typing.Optional[EnvironmentVariablesListRequestType]
            Filter by variable type

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariablesListResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.environment_variables.list(
                cursor="cursor",
                page_size=1,
                label="label",
                environment="environment",
                type="string",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            label=label,
            environment=environment,
            type=type,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self, *, request: EnvironmentVariablesCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> EnvironmentVariableResponse:
        """
        Create a new environment variable for the workspace

        Parameters
        ----------
        request : EnvironmentVariablesCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariableResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs
        from elevenlabs.environment_variables import (
            EnvironmentVariablesCreateRequestBody_String,
        )

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.environment_variables.create(
                request=EnvironmentVariablesCreateRequestBody_String(
                    label="label",
                    values={"key": "value"},
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def get(
        self, env_var_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EnvironmentVariableResponse:
        """
        Get a specific environment variable by ID

        Parameters
        ----------
        env_var_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariableResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.environment_variables.get(
                env_var_id="env_var_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(env_var_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        env_var_id: str,
        *,
        values: typing.Dict[str, typing.Optional[UpdateEnvironmentVariableRequestValuesValue]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EnvironmentVariableResponse:
        """
        Replace an environment variable's values. Use null to remove an environment (except production).

        Parameters
        ----------
        env_var_id : str

        values : typing.Dict[str, typing.Optional[UpdateEnvironmentVariableRequestValuesValue]]
            Values to replace. Set to null to remove an environment (except 'production').

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvironmentVariableResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.environment_variables.update(
                env_var_id="env_var_id",
                values={},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(env_var_id, values=values, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .forced_alignment_character_response_model import ForcedAlignmentCharacterResponseModel
from .forced_alignment_word_response_model import ForcedAlignmentWordResponseModel


class ForcedAlignmentResponseModel(UncheckedBaseModel):
    """
    Model representing the response from the aligner service.
    """

    characters: typing.List[ForcedAlignmentCharacterResponseModel] = pydantic.Field()
    """
    List of characters with their timing information.
    """

    words: typing.List[ForcedAlignmentWordResponseModel] = pydantic.Field()
    """
    List of words with their timing information.
    """

    loss: float = pydantic.Field()
    """
    The average alignment loss/confidence score for the entire transcript, calculated from all characters.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class GetWhatsAppAccountResponse(UncheckedBaseModel):
    business_account_id: str
    phone_number_id: str
    business_account_name: str
    phone_number_name: str
    phone_number: str
    assigned_agent_id: typing.Optional[str] = None
    enable_messaging: typing.Optional[bool] = None
    enable_audio_message_response: typing.Optional[bool] = None
    assigned_agent_name: typing.Optional[str] = None
    is_token_expired: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .... import core
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.edit_project_response_model import EditProjectResponseModel
from .raw_client import AsyncRawContentClient, RawContentClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ContentClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawContentClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawContentClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawContentClient
        """
        return self._raw_client

    def update(
        self,
        project_id: str,
        *,
        from_url: typing.Optional[str] = OMIT,
        from_document: typing.Optional[core.File] = OMIT,
        from_content_json: typing.Optional[str] = OMIT,
        auto_convert: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditProjectResponseModel:
        """
        Updates Studio project content.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        from_url : typing.Optional[str]
            An optional URL from which we will extract content to initialize the Studio project. If this is set, 'from_url' and 'from_content' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

        from_document : typing.Optional[core.File]
            See core.File for more documentation

        from_content_json : typing.Optional[str]

                An optional content to initialize the Studio project with. If this is set, 'from_url' and 'from_document' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

                Example:
                [{"name": "Chapter A", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "A", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "B", "type": "tts_node"}]}, {"sub_type": "h1", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "C", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "D", "type": "tts_node"}]}]}, {"name": "Chapter B", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "E", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "F", "type": "tts_node"}]}, {"sub_type": "h2", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "G", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "H", "type": "tts_node"}]}]}]


        auto_convert : typing.Optional[bool]
            Whether to auto convert the Studio project to audio or not.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditProjectResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.content.update(
            project_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.update(
            project_id,
            from_url=from_url,
            from_document=from_document,
            from_content_json=from_content_json,
            auto_convert=auto_convert,
            request_options=request_options,
        )
        return _response.data


class AsyncContentClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawContentClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawContentClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawContentClient
        """
        return self._raw_client

    async def update(
        self,
        project_id: str,
        *,
        from_url: typing.Optional[str] = OMIT,
        from_document: typing.Optional[core.File] = OMIT,
        from_content_json: typing.Optional[str] = OMIT,
        auto_convert: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditProjectResponseModel:
        """
        Updates Studio project content.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        from_url : typing.Optional[str]
            An optional URL from which we will extract content to initialize the Studio project. If this is set, 'from_url' and 'from_content' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

        from_document : typing.Optional[core.File]
            See core.File for more documentation

        from_content_json : typing.Optional[str]

                An optional content to initialize the Studio project with. If this is set, 'from_url' and 'from_document' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

                Example:
                [{"name": "Chapter A", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "A", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "B", "type": "tts_node"}]}, {"sub_type": "h1", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "C", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "D", "type": "tts_node"}]}]}, {"name": "Chapter B", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "E", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "F", "type": "tts_node"}]}, {"sub_type": "h2", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "G", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "H", "type": "tts_node"}]}]}]


        auto_convert : typing.Optional[bool]
            Whether to auto convert the Studio project to audio or not.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditProjectResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.content.update(
                project_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            project_id,
            from_url=from_url,
            from_document=from_document,
            from_content_json=from_content_json,
            auto_convert=auto_convert,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

DubbingListRequestDubbingStatusesItem = typing.Union[
    typing.Literal["queued", "preparing", "dubbing", "dubbed", "failed"], typing.Any
]
